from typing import Literal

from pathlib import Path

import click
import os

from ..session import Session
from .workspaces import Workspaces, Workspace


@click.group
# @click.option(
#     "-n", "--name", default=None, help="select an existing workspace by name."
# )
@click.pass_context
def dev(ctx):
    """Developer tools for the current workspace."""
    pass


def add_dev_commands(parent_group):
    parent_group.add_command(dev)


@dev.command()
@click.argument("index_name")
@click.pass_context
def create_local_index(ctx, index_name: str):
    """
    Create a package index folder in *home*, and set it as default index on
    the workspaces config.
    """
    session: Session = ctx.obj.session
    workspaces: Workspaces = ctx.obj.workspaces

    pi = session.home / index_name
    pi.mkdir(exist_ok=True)
    workspaces.config.default_index = "../../" + index_name
    workspaces.save_config()


@dev.command()
@click.option(
    "-i",
    "--index_name",
    help="Name of the local index. Default is to get it from Workspaces config.",
)
@click.option(
    "-p", "--project", help="Path of the project to build. Defaults to current path."
)
@click.option(
    "-B",
    "--builder",
    default="hatch",
    help='Builder to use, one of ["hatch", "uv", "build"] (defaults to "hatch").',
)
@click.pass_context
def build_to_local_index(
    ctx,
    index_name: str | None,
    project: str | None,
    builder: Literal["hatch", "uv", "build"] = "hatch",
):
    """
    Build the project in the current directory directly in
    the specified local pacakge index (a folder in the *home*)

    Note: `hatch` must be installed, or you must use the --builder option.
    """
    session: Session = ctx.obj.session
    workspaces: Workspaces = ctx.obj.workspaces
    workspace: Workspace = ctx.obj.workspace

    if workspace is not None:
        index = workspace.resolve_index(index_name)
    else:
        index = workspaces.resolve_index(index_name)

    if index is None:
        raise click.UsageError(
            f"No index_name specified and no default index configured. Use --index-name or `tgzr ws dev create-local-index INDEX_NAME` first."
        )

    if "://" in index:
        raise click.UsageError(
            f'The index "{index}" is not a local path. Select another one with --index-name or create one with `tgzr ws dev create-local-index path/to/index`.'
        )

    local_index = Path(index)
    if not local_index.exists():
        raise click.UsageError(
            f'The folder "{index}" does not exists. Select another one with --index-name or create one with `tgzr ws dev create-local-index path/to/index` first.'
        )

    if builder == "hatch":
        cwd = None
        if project is not None:
            cwd = Path(".").resolve()
            os.chdir(project)
        cmd = f"hatch build {local_index}"
        try:
            os.system(cmd)
        finally:
            if cwd is not None:
                os.chdir(cwd)

    elif builder == "uv":
        src_option = ""
        if project is not None:
            src_option = project
        cmd = f"uv build --out-dir {local_index} {src_option}"
        os.system(cmd)

    elif builder == "build":
        src_option = ""
        if project is not None:
            src_option = project
        cmd = f"python -m build --outdir {local_index} {src_option}"
        os.system(cmd)

    else:
        raise click.UsageError(f"Unsupported builder: {builder!r}")



from __future__ import annotations
from typing import Type

from pathlib import Path

from .config import BaseConfig, Field, SettingsConfigDict
from .workspaces.workspaces import WorkspacesConfig, WorkspaceConfig


class SessionConfig(BaseConfig):
    model_config = SettingsConfigDict(env_prefix="tgzr_")
    verbose: bool = Field(False, description="Tchatty logs")


class Session:

    def __init__(self):
        self._config_filename = ".tgzr"
        self._home: Path = Path.cwd().resolve()
        config_path = SessionConfig.find_config_file(self._home, self._config_filename)
        if config_path is not None:
            self._config = SessionConfig.from_file(config_path)
        else:
            self._config: SessionConfig = SessionConfig(
                _env_file=Path.cwd() / self._config_filename  # type: ignore No parameter named "_env_file"
            )

        self._known_config_types: list[Type[BaseConfig]] = []

        self.declare_config_type(SessionConfig)
        self.declare_config_type(WorkspacesConfig)
        self.declare_config_type(WorkspaceConfig)

    @property
    def home(self) -> Path:
        if self._home is None:
            raise ValueError("Home not set. Please use `set_home(path) first.`")
        return self._home

    def set_home(
        self, path: str | Path, ensure_config_found: bool = True
    ) -> Path | None:
        """Returns the path of the config loaded or None."""
        path = Path(path).resolve()
        config_path = SessionConfig.find_config_file(path, self._config_filename)
        if config_path is None:
            if ensure_config_found:
                raise FileNotFoundError(
                    f"Could not find a {self._config_filename} file under {path!r} and ancestors. "
                    "(Use `ensure_config_found=False` to bypass this error.)"
                )
            self._home = path
            return None
        else:
            self._home = config_path.parent
            self._config = SessionConfig.from_file(config_path)
            return config_path

    @property
    def config(self) -> SessionConfig:
        if self._config is None:
            raise ValueError("Config not set. Please use `set_home(path) first.`")
        return self._config

    def save_config(self) -> Path:
        """Returns the path of the saved file."""
        return self.write_config_file(None, allow_overwrite=True)

    def write_config_file(
        self, path: str | Path | None, allow_overwrite: bool = False
    ) -> Path:
        """Returns the path of the saved file."""
        if path is None:
            path = self._home / self._config_filename
        path = Path(path).resolve()
        self._config.to_file(
            path,
            allow_overwrite=allow_overwrite,
            header_text="tgzr session config",
        )
        return path

    def declare_config_type(self, config_type: Type[BaseConfig]):
        """
        Declare the config type as used in the session.
        This is used for documentation and cli help.
        """
        self._known_config_types.append(config_type)

    def get_config_env_vars(self) -> list[tuple[str, list[tuple[str, str | None]]]]:
        """
        Returns a list of (name, description) for each config
        declared with `declare_config_type()`
        """
        config_env_vars = []
        for config_type in self._known_config_types:
            config_env_vars.append((config_type.__name__, config_type.used_env_vars()))
        return config_env_vars



from typing import Literal

import click
import rich

from tgzr.cli.add_plugins import find_group
from ..session import Session

from .workspaces import Workspaces, Workspace

from .dev_cli import add_dev_commands


def install_plugin(group: click.Group):
    group.add_command(ws)
    help = find_group(group, named="help")
    if help is None:
        raise Exception("`help` group not found, cannot install ws help commands!")
    help.add_command(ws_help)


@click.command("ws")
def ws_help():
    rich.print("Sorry, this help is not written yet :p")


@click.group(help="Manage tgzr workspaces")
@click.option(
    "-n", "--name", default=None, help="select an existing workspace by name."
)
@click.pass_context
def ws(ctx, name: str | None = None):
    session: Session = ctx.obj.session
    workspaces = Workspaces(parent=session.home)
    if name is not None:
        try:
            workspace = workspaces.get_workspace(name)
        except FileNotFoundError:
            raise click.UsageError(
                f'Workspace with name {name!r} does not exist in "{workspaces.home}".'
            )
    else:
        workspace = workspaces.get_default_workspace(ensure_exists=False)
    ctx.obj.workspaces = workspaces
    ctx.obj.workspace = workspace  # may be None


# register the dev command group:
add_dev_commands(ws)


@ws.command()
@click.argument("name")
@click.option("--allow-existing", is_flag=True, default=False)
@click.option(
    "--index", default=None, help="Override the default index (used for seed installs)."
)
@click.option(
    "-N",
    "--not-default",
    is_flag=True,
    default=False,
    help="Do not make this new workspace the default one.",
)
@click.pass_context
def create(
    ctx,
    name,
    allow_existing: bool = False,
    index: str | None = None,
    not_default: bool = True,
):
    """
    Create a new workspace with name "NAME".
    """
    session: Session = ctx.obj.session
    workspaces: Workspaces = ctx.obj.workspaces
    workspace = workspaces.get_workspace(name, ensure_exists=False)
    if workspace.exists():
        if not allow_existing:
            raise click.UsageError(
                f'A workspace {name!r} already exists in "{workspaces.home}". Use --allow-existing to update it.'
            )

    workspace.create(index=index)
    click.echo(f"Workspace {name} created.")

    # Ensure the main config is written so the session home can
    # be discovered:
    session.save_config()

    if not_default:
        return

    workspaces.set_default_workspace(name)
    ctx.obj.workspace = workspace
    click.echo(f"Workspace {name} set as default workspace.")


@ws.group(
    # invoke_without_command=True,
    help="Manage the workspaces configuration (Not the configuration for the current workspace!)",
)
@click.pass_context
def config(ctx):
    pass


@config.command("show")
@click.pass_context
def config_show(ctx):
    """
    Show the workspaces configuration.
    """
    workspaces: Workspaces = ctx.obj.workspaces
    rich.print(workspaces.config)


@config.command("set")
@click.pass_context
@click.option(
    "--default-index",
    help=(
        "Set the url of the default package index, like "
        '"https://pypi.org/simple", "/path/to/folder" or "./path/relative/to/workspace". '
        'Use "" to unset.'
    ),
)
def config_set(ctx, **kwargs):
    """
    Set config field(s).
    """
    workspaces: Workspaces = ctx.obj.workspaces

    not_None_options = [n for n, v in kwargs.items() if v is not None]
    if not not_None_options:
        raise click.UsageError("Please specify at least one option.")

    # print(kwargs)

    if (default_index := kwargs["default_index"]) is not None:
        if not default_index:
            default_index = None
        workspaces.config.default_index = default_index

    path = workspaces.save_config()
    click.echo(f"Config saved: {path}")


@ws.command()
@click.pass_context
def ls(ctx):
    """
    List the existing workspaces.
    """
    workspaces: Workspaces = ctx.obj.workspaces
    for ws in workspaces.get_workspaces():
        click.echo(f"  {ws.name} ({ws.path})")


@ws.command()
@click.pass_context
def show(ctx):
    """Show the current workspace config."""
    workspace: Workspace = ctx.obj.workspace
    if workspace is None:
        click.echo(
            'No workspace selected, use "tgzr ws --name NAME ..." to specify a workspace, or "tgzr ws select" to set the default one.'
        )
        return
    click.echo(f'Configuration for workspace "{workspace.name}" ({workspace.path}):')
    rich.print(workspace.config)


@ws.command()
@click.pass_context
def select(ctx):
    """
    Set the default workspace.
    """
    workspaces: Workspaces = ctx.obj.workspaces
    default_workspace_name = workspaces.default_workspace_name()
    workspace: Workspace = ctx.obj.workspace
    if workspace is not None and workspace.name == default_workspace_name:
        click.echo(f"The default workspace was already {default_workspace_name}.")
    else:
        workspaces.set_default_workspace(workspace.name)


@ws.command()
@click.pass_context
def switch(ctx):
    """
    Change the default workspace.
    """
    workspaces: Workspaces = ctx.obj.workspaces
    workspace_names = workspaces.workspace_names()
    if not workspace_names:
        click.echo('No workspace declared yet. Use "tgzr ws add" to add workspaces.')
        return

    click.echo("0 <None>")
    for i, name in enumerate(workspace_names, start=1):
        click.echo(f"{i} {name}")
    value = click.prompt("Enter workspace")
    if value.strip() == "0":
        workspaces.set_default_workspace(None)
        return

    name = None
    try:
        index = int(value)
    except ValueError:
        if value in workspace_names:
            name = value
    else:
        try:
            name = workspace_names[index - 1]
        except:
            pass

    if name is None:
        click.echo("Nope ¯\\_(ツ)_/¯")
    else:
        workspaces.set_default_workspace(name)


@ws.command()
@click.pass_context
@click.option(
    "--default-index",
    help=(
        "Set the url of the default package index, like "
        '"https://pypi.org/simple", "/path/to/folder" or "./path/relative/to/workspace". '
        'Use "" to unset.'
    ),
)
def set(ctx, **kwargs):
    """
    Set config field(s) on the current workspace.
    """
    workspace: Workspace = ctx.obj.workspace

    not_None_options = [n for n, v in kwargs.items() if v is not None]
    if not not_None_options:
        raise click.UsageError("Please specify at least one option.")

    # print(kwargs)

    if (default_index := kwargs["default_index"]) is not None:
        if not default_index:
            default_index = None
        workspace.config.default_index = default_index

    path = workspace.save_config()
    click.echo(f"Config saved: {path}")


@ws.command(
    context_settings=dict(
        ignore_unknown_options=True,
    )
)
@click.pass_context
@click.argument(
    "cmd_name",
)
@click.argument("cmd_args", nargs=-1, type=click.UNPROCESSED)
def run(ctx, cmd_name: str, cmd_args: list[str], tgzr_project: str | None):
    """
    Run a master cmd in the current workspace.
    """
    workspace: Workspace = ctx.obj.workspace
    workspace.run_cmd(cmd_name, cmd_args, project_name=None)


@ws.command
@click.option(
    "-s",
    "--shell",
    default="auto",
    help='The shell to use, one of ["xonsh", "cmd", "powershell", "auto"] (defaults to "auto").',
)
@click.pass_context
def shell(
    ctx,
    shell: Literal["xonsh", "cmd", "powershell", "auto"] = "auto",
):
    """
    Open a master shell in the current workspace.
    """
    workspace: Workspace = ctx.obj.workspace
    workspace.shell(shell_type=shell, project_name=None)


@ws.group(
    help="Manage the project in the current workspaces",
)
@click.argument("name")
@click.pass_context
def project(ctx, name):
    ctx.obj.project_name = name


@project.command(
    "run",
    context_settings=dict(
        ignore_unknown_options=True,
    ),
)
@click.pass_context
@click.argument(
    "cmd_name",
)
@click.argument("cmd_args", nargs=-1, type=click.UNPROCESSED)
def project_run(ctx, cmd_name: str, cmd_args: list[str]):
    """
    Run a cmd in a project of the current workspace.
    """
    workspace: Workspace = ctx.obj.workspace
    project_name = ctx.obj.project_name
    workspace.run_cmd(cmd_name, cmd_args, project_name)



from pathlib import Path

import click
import rich

from tgzr.cli.session import Session


def install_plugin(group: click.Group):
    group.add_command(config)


@click.group(help="Manage tgzr cli configs.")
def config():
    pass


@config.command()
@click.pass_context
def show(ctx):
    session: Session = ctx.obj.session
    rich.print(session.config)


@config.command()
@click.pass_context
@click.option("-o", "--output", help='The file to save. Defaults to "home/.tgzr".')
@click.option("--force", is_flag=True)
def save(ctx, output: Path | None = None, force: bool = False):
    session: Session = ctx.obj.session

    try:
        session.write_config_file(output, allow_overwrite=force)
    except FileExistsError as err:
        click.echo(f"File {err} already exists! (Use --force to allow overwrite.)")



import sys
import os
import platform
from pathlib import Path


class Venv:
    def __init__(self, path: Path) -> None:
        self._path = path
        if platform.system() == "Windows":
            self._bin_path = self._path / "Scripts"
        else:
            self._bin_path = self._path / "bin"

    @property
    def path(self) -> Path:
        return self._path

    def exists(self) -> bool:
        return self.path.exists() and self._bin_path.exists()

    def create(self, with_python_exe: str):
        cmd = f"{with_python_exe} -m venv {self.path}"
        print(f"Creating venv {self.path}: {cmd}")
        os.system(cmd)

    def get_exe(self, name: str):
        exe = self._bin_path / name
        if platform.system() == "Windows":
            # the '.exe' suffix is not needed to execute
            # but the caller may want to use `exe.exists()``
            # so we need to add the extension if on windows :/
            exe_suffix = exe.with_suffix(".exe")
            if exe_suffix.exists():
                # special case for .bat files
                exe = exe_suffix
        return exe

    def install_uv(self):
        return self.install_package("uv", use_uv=False)

    def install_package(
        self,
        requirement,
        update: bool = True,
        use_uv: bool = True,
        index: str | None = None,
    ) -> bool:
        """
        Returns True if the package has been successfully installed.

        if `index` is given, it will be passed to pip:
            - as -f/--find-links it it is a path (Note: relative path are discouraged).
            - as -i/--index-url otherwise
        """
        if use_uv:
            # NB: we're using python -m uv instead of uv directly so that uv pip installs in the same env
            # see "If uv is installed in a Python environment" in https://docs.astral.sh/uv/pip/environments/#using-arbitrary-python-environments
            exe = f'{self.get_exe("python")} -m uv pip'
        else:
            exe = f'{self.get_exe("python")} -m pip'

        index_flags = ""
        if index is not None:
            if "://" in index:
                index_flags = f"--index-url {index}"
            else:
                index_flags = f"--find-links {index}"

        update_flag = ""
        if update:
            update_flag = "-U"

        cmd = f"{exe} install {index_flags} {update_flag} {requirement}"
        print(f"Installing package {requirement}: {cmd}")
        ret = os.system(cmd)
        return not ret

    def execute_cmd(self, cmd: str) -> bool:
        """
        Returns True if the command was been successfully executed.
        """
        print("Executing venv cmd:", cmd)
        ret = os.system(cmd)
        return not ret

    def run_cmd(self, cmd_name: str, cmd_args: list[str]) -> bool:
        """
        Returns True if the command was been successfully executed.
        """
        # TODO: thread this.
        exe = self.get_exe(cmd_name)
        cmd = str(exe) + " " + " ".join(cmd_args)
        return self.execute_cmd(cmd)

    def get_cmd_names(self) -> list[str]:
        raise NotImplementedError()


class PackageManager:
    def __init__(self, root: Path) -> None:
        self._root = root

    @property
    def root(self) -> Path:
        return self._root

    def get_venv(self, venv_name: str, group: str) -> Venv:
        return Venv(self.root / group / venv_name)

    def _create_bat_shortcut(self, exe_path: Path | str, shortcut_path: Path | str):
        exe_path = Path(exe_path)
        if not exe_path.is_absolute() and not str(exe_path).startswith("./"):
            exe_path = f".\\{exe_path}"

        shortcut_path = Path(shortcut_path)
        if shortcut_path.suffix != ".bat":
            shortcut_path = f"{shortcut_path}.bat"

        content = [
            "@echo off",
            f"REM Shortcut to {exe_path}",
            "",
            f"{exe_path} %*",
        ]
        with open(shortcut_path, "w") as fp:
            fp.write("\n".join(content))

    def create_shortcut(
        self, exe_path: Path | str, shortcut_path: Path | str, relative: bool = True
    ):
        """
        Create a shortcut to exe_path at shortcut_path.

        If `relative` is True, `exe_path` will be modified to be relative
        to `shortcut_path`.

        The shortcut will be a symlink to the exe, unless the OS does not
        allow it in which case a .bat file is created.
        """
        if relative:
            exe_path = Path(exe_path).relative_to(Path(shortcut_path).parent)

        if platform.system() == "Linux":
            os.symlink(exe_path, shortcut_path, target_is_directory=False)
        else:
            self._create_bat_shortcut(exe_path, shortcut_path)

    def create_venv(self, venv_name: str, group: str, exist_ok: bool = False) -> Venv:
        venv = self.get_venv(venv_name, group)
        if venv.exists() and not exist_ok:
            raise ValueError(f"Virtual Env {venv.path} already exists!")

        python_exe = sys.executable
        # TODO: assert sys.executable works with pyinstaller
        # TODO: detect if we can use uv instead of "python -m venv" and use it when possible.
        venv.create(python_exe)
        venv.install_uv()
        return venv



from __future__ import annotations
from typing import Literal

from pathlib import Path
import platform

from ..config import BaseConfig, SettingsConfigDict, Field

# TODO: put this in a dedicated package and add it as dependency:
# from tgzr.package_manager import PackageManager
from tgzr.shell.package_manager import PackageManager


class WorkspacesConfig(BaseConfig):
    model_config = SettingsConfigDict(env_prefix="tgzr_ws_")

    default_workspace_name: str | None = Field(
        None, description="Name of the default workspace"
    )
    default_index: str | None = Field(
        None,
        description="Default package index for all workspaces. Use None for https://pypi.org/simple",
    )


class Workspaces:
    def __init__(self, parent: Path | str):
        parent = Path(parent)
        if not parent.is_dir():
            raise ValueError(
                f'The path "{parent}" is not a directory, cannot instantiate Workspaces here.'
            )
        self._home = (parent / "Workspaces").resolve()
        if not self._home.exists():
            self._home.mkdir()
        self._config_path = self._home / ".tgzr_workspaces"
        self._config = WorkspacesConfig.from_file(self._config_path)

    @property
    def home(self) -> Path:
        return self._home

    @property
    def config(self) -> WorkspacesConfig:
        return self._config

    def workspace_names(self) -> list[str]:
        """
        Return the names of the existing workspaces.
        """
        if not self._home.exists():
            return []
        names = []
        for path in self._home.iterdir():
            config_path = Workspace.config_path(self, path.name)
            if config_path.exists():
                names.append(path.name)
        return sorted(names)

    def get_workspaces(self) -> list[Workspace]:
        return [
            self.get_workspace(name, ensure_exists=False)
            for name in self.workspace_names()
        ]

    def set_default_workspace(
        self, workspace_name: str | None, ensure_exists: bool = True
    ) -> None:
        if workspace_name is None:
            self._config.default_workspace_name = None
        else:
            if ensure_exists:
                # will raise if it does not exists:
                self.get_workspace(workspace_name, ensure_exists=True)
            self._config.default_workspace_name = workspace_name
        self.save_config()

    def save_config(self) -> Path:
        """Returns the path of the saved file."""
        self._config.to_file(
            self._config_path,
            allow_overwrite=True,
            header_text="tgzr workspaces config",
        )
        return self._config_path

    def default_workspace_name(self) -> str | None:
        return self.config.default_workspace_name

    def get_default_workspace(self, ensure_exists: bool = True) -> Workspace | None:
        name = self._config.default_workspace_name
        if name is None:
            return None
        return self.get_workspace(name, ensure_exists)

    def get_workspace(self, name: str, ensure_exists: bool = True) -> Workspace:
        ws = Workspace(self, name)
        if ensure_exists and not ws.exists():
            raise FileNotFoundError
        return ws

    def resolve_index(
        self, index: str | None, relative_to: Path | None = None
    ) -> str | None:
        """
        Returns the path of an index, making it absolute in case
        of relative local path.
        If `relative_to` is not provided, `self.home` is used.

        If index is None, the default index will be returned.
        If no default index is set, None is returned.
        """
        index = index or self.config.default_index
        if index is None:
            return None
        if not "://" in index:
            local_index = Path(index)
            if not local_index.is_absolute():
                relative_to = relative_to or self.home
                index = str((relative_to / local_index).resolve())
        return index


class WorkspaceConfig(BaseConfig):
    model_config = SettingsConfigDict(env_prefix="tgzr_ws_default_")

    default_index: str | None = Field(
        None,
        description="Default package index for this workspace. Use None to fallback to Worksapces config",
    )


class Workspace:
    @classmethod
    def config_path(cls, workspaces: Workspaces, name: str) -> Path:
        return workspaces.home / name / ".tgzr_workspace"

    def __init__(self, workspaces: Workspaces, name: str) -> None:
        self._workspaces = workspaces
        self._name = name
        self._config_path = self.config_path(workspaces, name)
        self._config = WorkspaceConfig.from_file(self._config_path)

        self._tgzr_venv_group = "_"
        self._tgzr_venv_name = "tgzr"
        self._projects_venv_group = "projects"

    @property
    def config(self) -> WorkspaceConfig:
        return self._config

    @property
    def name(self) -> str:
        return self._name

    @property
    def workspaces(self) -> Workspaces:
        return self._workspaces

    @property
    def path(self) -> Path:
        return self.workspaces.home / self.name

    # @property
    # def default_index(self) -> str | None:
    #     return (
    #         self.config.default_index or self._workspaces.config.default_index or None
    #     )

    def resolve_index(self, index: str | None) -> str | None:
        """
        Returns the path of an index, making it absolute in case
        of relative local path.

        If index is None, the default index will be returned.
        If no default index in found on the Workspace nor on the Workspaces,
        None is returned.
        """
        index = index or self.config.default_index
        return self._workspaces.resolve_index(index, relative_to=self.path)

    def save_config(self) -> Path:
        """Returns the path of the saved file."""
        self._config.to_file(
            self._config_path,
            allow_overwrite=True,
            header_text="tgzr workspaces config",
        )
        return self._config_path

    def exists(self) -> bool:
        return self.path.exists()

    def ensure_exists(self):
        if not self.exists:
            self.create()

    def create(self, index: str | None = None):
        """
        Create (or update) this workspace and seed it with the `tgzr.cli` and `uv` packages.
        If `index` is None, the parent workspaces' default index is used.
        """
        self.path.mkdir(exist_ok=True)
        self.save_config()

        venvs = self.path / "venvs"
        pm = PackageManager(venvs)
        venv = pm.create_venv(
            self._tgzr_venv_name, group=self._tgzr_venv_group, exist_ok=True
        )

        index = self.resolve_index(index)
        ok = venv.install_package("tgzr.cli", index=index, update=True)
        if not ok:
            raise Exception("Could not install tgzr in workspace :( Aborting.")

        tgzr_exe = venv.get_exe("tgzr")
        tgzr_shortcut = self.path / "tgzr"
        if tgzr_shortcut.exists():
            tgzr_shortcut.unlink()
        pm.create_shortcut(tgzr_exe, tgzr_shortcut, relative=True)

    def run_cmd(
        self, cmd_name: str, cmd_args: list[str], project_name: str | None = None
    ):
        """
        Run a command in the venv for the given project.
        If project_name is not provided, the "master" venv is used ('_/tgzr')

        Returns True if the command was executed successfuly (according to process return int).
        """
        if project_name is None:
            group = self._tgzr_venv_group
            venv_name = self._tgzr_venv_name
        else:
            group = self._projects_venv_group
            venv_name = project_name

        venvs = self.path / "venvs"
        pm = PackageManager(venvs)
        venv = pm.get_venv(venv_name, group)
        return venv.run_cmd(cmd_name, cmd_args)

    def shell(
        self,
        shell_type: Literal["xonsh", "cmd", "powershell", "auto"],
        project_name: str | None = None,
    ):
        """
        Open a shell in the venv for the given project.
        If project_name is not provided, the "master" venv is used ('_/tgzr')

        Returns True if the command was executed successfuly (according to process return int).
        """
        if project_name is None:
            group = self._tgzr_venv_group
            venv_name = self._tgzr_venv_name
        else:
            group = self._projects_venv_group
            venv_name = project_name

        venvs = self.path / "venvs"
        pm = PackageManager(venvs)
        venv = pm.get_venv(venv_name, group)

        if shell_type == "auto":
            if platform.system() == "Windows":
                shell_type = "cmd"
            else:
                shell_type = "xonsh"

        if shell_type == "xonsh":
            xonsh = venv.get_exe("xonsh")
            if not xonsh.exists():
                venv.install_package("xonsh[full]")
            uv = venv.get_exe("uv")
            cmd = f"{uv} run xonsh"
        else:
            activate = str(venv.get_exe("activate")).replace("/", "\\")
            cmd = f"start cmd /k {activate}"

        return venv.execute_cmd(cmd)



from pathlib import Path

import click
import rich
import rich.table

from tgzr.cli.session import Session


def install_plugin(group: click.Group):
    group.add_command(help)


@click.group(help="Documentations and tooltips.")
def help():
    pass


@help.command()
@click.pass_context
def env(ctx):
    session: Session = ctx.obj.session

    config_env_vars = session.get_config_env_vars()

    for name, env_vars in config_env_vars:
        table = rich.table.Table("Name", "Description", title=name)

        for name, description in env_vars:
            table.add_row(name, description or "")

        rich.print("\n", table)
